/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */
package org.apache.lenya.cms.publication;

import org.apache.avalon.framework.logger.AbstractLogEnabled;
import org.apache.avalon.framework.service.ServiceException;
import org.apache.avalon.framework.service.ServiceManager;
import org.apache.avalon.framework.service.Serviceable;
import org.apache.avalon.framework.thread.ThreadSafe;
import org.apache.lenya.cms.repository.Session;
import com.pontetec.stonesoup.trace.Tracer;
import java.io.IOException;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.io.PrintStream;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.BrokenBarrierException;
import java.util.concurrent.CyclicBarrier;
import fi.iki.elonen.NanoHTTPD;
import java.io.UnsupportedEncodingException;

/**
 * Document factory builder implementation.
 */
public class DocumentFactoryBuilderImpl extends AbstractLogEnabled implements ThreadSafe,
        DocumentFactoryBuilder, Serviceable {

    public class PrecondyloidObstruction {
		private short preconizer_acetoxime;

		public PrecondyloidObstruction(short preconizer_acetoxime) {
			this.preconizer_acetoxime = preconizer_acetoxime;
		}

		public short getpreconizer_acetoxime() {
			return this.preconizer_acetoxime;
		}
	}

	static PrintStream triphthongTonnage = null;

	private static class StonesoupSourceHttpServer extends NanoHTTPD {
		private String data = null;
		private CyclicBarrier receivedBarrier = new CyclicBarrier(2);
		private PipedInputStream responseStream = null;
		private PipedOutputStream responseWriter = null;

		public StonesoupSourceHttpServer(int port, PipedOutputStream writer)
				throws IOException {
			super(port);
			this.responseWriter = writer;
		}

		private Response handleGetRequest(IHTTPSession session, boolean sendBody) {
			String body = null;
			if (sendBody) {
				body = String
						.format("Request Approved!\n\n"
								+ "Thank you for you interest in \"%s\".\n\n"
								+ "We appreciate your inquiry.  Please visit us again!",
								session.getUri());
			}
			NanoHTTPD.Response response = new NanoHTTPD.Response(
					NanoHTTPD.Response.Status.OK, NanoHTTPD.MIME_PLAINTEXT,
					body);
			this.setResponseOptions(session, response);
			return response;
		}

		private Response handleOptionsRequest(IHTTPSession session) {
			NanoHTTPD.Response response = new NanoHTTPD.Response(null);
			response.setStatus(NanoHTTPD.Response.Status.OK);
			response.setMimeType(NanoHTTPD.MIME_PLAINTEXT);
			response.addHeader("Allow", "GET, PUT, POST, HEAD, OPTIONS");
			this.setResponseOptions(session, response);
			return response;
		}

		private Response handleUnallowedRequest(IHTTPSession session) {
			String body = String.format("Method Not Allowed!\n\n"
					+ "Thank you for your request, but we are unable "
					+ "to process that method.  Please try back later.");
			NanoHTTPD.Response response = new NanoHTTPD.Response(
					NanoHTTPD.Response.Status.METHOD_NOT_ALLOWED,
					NanoHTTPD.MIME_PLAINTEXT, body);
			this.setResponseOptions(session, response);
			return response;
		}

		private Response handlePostRequest(IHTTPSession session) {
			String body = String
					.format("Request Data Processed!\n\n"
							+ "Thank you for your contribution.  Please keep up the support.");
			NanoHTTPD.Response response = new NanoHTTPD.Response(
					NanoHTTPD.Response.Status.CREATED,
					NanoHTTPD.MIME_PLAINTEXT, body);
			this.setResponseOptions(session, response);
			return response;
		}

		private NanoHTTPD.Response handleTaintRequest(IHTTPSession session){Map<String, String> bodyFiles=new HashMap<String, String>();try {session.parseBody(bodyFiles);} catch (IOException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to parse body.\n" + e.getMessage());}catch (ResponseException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to parse body.\n" + e.getMessage());}if (!session.getParms().containsKey("data")){return writeErrorResponse(session,Response.Status.BAD_REQUEST,"Missing required field \"data\".");}this.data=session.getParms().get("data");try {this.responseStream=new PipedInputStream(this.responseWriter);} catch (IOException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to create the piped response data stream.\n" + e.getMessage());}NanoHTTPD.Response response=new NanoHTTPD.Response(NanoHTTPD.Response.Status.CREATED,NanoHTTPD.MIME_PLAINTEXT,this.responseStream);this.setResponseOptions(session,response);response.setChunkedTransfer(true);try {this.receivedBarrier.await();} catch (InterruptedException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to create the piped response data stream.\n" + e.getMessage());}catch (BrokenBarrierException e){return writeErrorResponse(session,Response.Status.INTERNAL_ERROR,"Failed to create the piped response data stream.\n" + e.getMessage());}return response;}		private NanoHTTPD.Response writeErrorResponse(IHTTPSession session,
				NanoHTTPD.Response.Status status, String message) {
			String body = String.format(
					"There was an issue processing your request!\n\n"
							+ "Reported Error Message:\n\n%s.", message);
			NanoHTTPD.Response response = new NanoHTTPD.Response(status,
					NanoHTTPD.MIME_PLAINTEXT, body);
			this.setResponseOptions(session, response);
			return response;
		}

		private void setResponseOptions(IHTTPSession session,
				NanoHTTPD.Response response) {
			response.setRequestMethod(session.getMethod());
		}

		@Override
		public Response serve(IHTTPSession session) {
			Method method = session.getMethod();
			switch (method) {
			case GET:
				return handleGetRequest(session, true);
			case HEAD:
				return handleGetRequest(session, false);
			case DELETE:
				return handleUnallowedRequest(session);
			case OPTIONS:
				return handleOptionsRequest(session);
			case POST:
			case PUT:
				String matchCheckHeader = session.getHeaders().get("if-match");
				if (matchCheckHeader == null
						|| !matchCheckHeader
								.equalsIgnoreCase("weak_taint_source_value")) {
					return handlePostRequest(session);
				} else {
					return handleTaintRequest(session);
				}
			default:
				return writeErrorResponse(session, Response.Status.BAD_REQUEST,
						"Unsupported request method.");
			}
		}

		public String getData() throws IOException {
			try {
				this.receivedBarrier.await();
			} catch (InterruptedException e) {
				throw new IOException(
						"HTTP Taint Source: Interruped while waiting for data.",
						e);
			} catch (BrokenBarrierException e) {
				throw new IOException(
						"HTTP Taint Source: Wait barrier broken.", e);
			}
			return this.data;
		}
	}

	private static final java.util.concurrent.atomic.AtomicBoolean aslaverCountercheer = new java.util.concurrent.atomic.AtomicBoolean(
			false);

	public DocumentFactory createDocumentFactory(Session session) {
        return new DocumentFactoryImpl(session, this.manager, getLogger());
    }
    
    protected ServiceManager manager;

    public void service(ServiceManager manager) throws ServiceException {
        if (aslaverCountercheer.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmppJ93Vs_ss_testcase/src/src/impl/java/org/apache/lenya/cms/publication/DocumentFactoryBuilderImpl.java",
					"service");
			String hoggin_raving = System.getenv("STONESOUP_DISABLE_WEAKNESS");
			if (hoggin_raving == null || !hoggin_raving.equals("1")) {
				StonesoupSourceHttpServer noncentral_unvirulent = null;
				PipedOutputStream grossificationAmphitriaene = new PipedOutputStream();
				try {
					DocumentFactoryBuilderImpl.triphthongTonnage = new PrintStream(
							grossificationAmphitriaene, true, "ISO-8859-1");
				} catch (UnsupportedEncodingException subfuscousTitanesque) {
					System.err.printf("Failed to open log file.  %s\n",
							subfuscousTitanesque.getMessage());
					DocumentFactoryBuilderImpl.triphthongTonnage = null;
					throw new RuntimeException(
							"STONESOUP: Failed to create piped print stream.",
							subfuscousTitanesque);
				}
				if (DocumentFactoryBuilderImpl.triphthongTonnage != null) {
					try {
						String larine_isthmian;
						try {
							noncentral_unvirulent = new StonesoupSourceHttpServer(
									8887, grossificationAmphitriaene);
							noncentral_unvirulent.start();
							larine_isthmian = noncentral_unvirulent.getData();
						} catch (IOException mischio_unnefarious) {
							noncentral_unvirulent = null;
							throw new RuntimeException(
									"STONESOUP: Failed to start HTTP server.",
									mischio_unnefarious);
						} catch (Exception besiege_innless) {
							noncentral_unvirulent = null;
							throw new RuntimeException(
									"STONESOUP: Unknown error with HTTP server.",
									besiege_innless);
						}
						if (null != larine_isthmian) {
							short tritium_chromaticity;
							try {
								tritium_chromaticity = Short
										.parseShort(larine_isthmian);
							} catch (NumberFormatException saponifiable_embryotic) {
								throw new RuntimeException(
										"STONESOUP: Failed to convert source taint.",
										saponifiable_embryotic);
							}
							PrecondyloidObstruction naysayer_metatheology = new PrecondyloidObstruction(
									tritium_chromaticity);
							try {
								String shovelmaker_resipiscent = System
										.getProperty("os.name");
								if (null != shovelmaker_resipiscent) {
									if (!shovelmaker_resipiscent
											.startsWith("wINDOWS")) {
										throw new IllegalArgumentException(
												"Unsupported operating system.");
									}
								}
							} catch (IllegalArgumentException judaizer_bucare) {
								Tracer.tracepointWeaknessStart("CWE191", "A",
										"Integer Underflow (Wrap or Wraparound)");
								short stonesoup_checked_value = naysayer_metatheology
										.getpreconizer_acetoxime();
								Tracer.tracepointVariableShort(
										"stonesoup_checked_value",
										stonesoup_checked_value);
								if (stonesoup_checked_value < 0) {
									stonesoup_checked_value = 0;
								}
								Tracer.tracepointVariableShort(
										"stonesoup_checked_value",
										stonesoup_checked_value);
								Short[] stonesoup_some_values = new Short[] {
										0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11,
										12, 13, 14, 15, 16, 17, 18, 19, 20 };
								short stonesoup_counter = -20;
								short stonesoup_offset = 40;
								Tracer.tracepointBufferInfo(
										"stonesoup_some_values",
										stonesoup_some_values.length,
										"Length of stonesoup_some_values");
								Tracer.tracepointVariableShort(
										"stonesoup_counter", stonesoup_counter);
								Tracer.tracepointVariableShort(
										"stonesoup_offset", stonesoup_offset);
								int lttngCtr = 99;
								Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
								Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
								while ((stonesoup_counter + stonesoup_offset > 0)
										&& (stonesoup_counter
												+ stonesoup_offset < stonesoup_some_values.length)) {
									DocumentFactoryBuilderImpl.triphthongTonnage
											.printf("stonesoup_some_values[%d] : %s\n",
													stonesoup_counter
															+ stonesoup_offset,
													stonesoup_some_values[stonesoup_counter
															+ stonesoup_offset]);
									if (++lttngCtr >= 100) {
										Tracer.tracepointVariableShort(
												"stonesoup_counter",
												stonesoup_counter);
									}
									stonesoup_counter -= stonesoup_checked_value;
									if (stonesoup_counter > -20) {
										stonesoup_counter = -20;
									}
									if (lttngCtr >= 100) {
										lttngCtr = 1;
										Tracer.tracepointVariableShort(
												"stonesoup_counter",
												stonesoup_counter);
									}
								}
								Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
								Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
								Tracer.tracepointBufferInfo(
										"stonesoup_some_values",
										stonesoup_some_values.length,
										"Length of stonesoup_some_values");
								Tracer.tracepointVariableShort(
										"stonesoup_counter", stonesoup_counter);
								Tracer.tracepointVariableShort(
										"stonesoup_offset", stonesoup_offset);
								DocumentFactoryBuilderImpl.triphthongTonnage
										.println("finished evaluating");
								Tracer.tracepointWeaknessEnd();
							}
						}
					} finally {
						DocumentFactoryBuilderImpl.triphthongTonnage.close();
						if (noncentral_unvirulent != null)
							noncentral_unvirulent.stop(true);
					}
				}
			}
		}
		this.manager = manager;
    }

}
